#ifndef display_h_included
#define display_h_included

#include "config.h"

// avoid flickering of the display:
#define TIME_BETWEEN_DISPLAY_UPDATES_ms 200
#define BLINK_ON_OFF_TIME_ms 1000


#define MAX_CLOCK_SPEED_LEN 8
#define MAX_CLOCK_WEEKDAY_LEN 4
#define MAX_NUMBER_CLIENTS_DISPLAYED 5
#define MAX_LOG_MESSAGE_LEN 30
#define MAX_NUMBER_LOG_LINES 5

enum ScreenMode { NoScreen, BootSequenceScreen, LogScreen, DashboardScreen };

class Display {
public:
  Display();
  void begin(void);
  bool showBootSequenceFinished(unsigned int ms_per_step); // call this in your loop function, parameter ms_per_step controls the speed by naming the milliseconds each step should take
  void showDashboard(void);
  void setClockName(const char *name) { strncpy(clockName, name, MAX_CLOCK_NAME_LEN); };
  void setClockSpeed(const char *speed) { strncpy(clockSpeed, speed, MAX_CLOCK_SPEED_LEN); };
  void setTime(int hour, int minute) {
    clockHour = hour;
    clockMinute = minute;
    Serial.print("display: new time "); Serial.print(clockHour); Serial.print(":"); Serial.println(clockMinute);
  };
  //void setClockSpeed(int _msPerModelSecond) { msPerModelSecond = _msPerModelSecond; setClockSpeed("x"); };
  void setClockWeekday(const char *weekday) { strncpy(clockWeekday, weekday, MAX_CLOCK_WEEKDAY_LEN); };
  void setClockHalted(bool halted) { clockHalted = halted; };
  void setNumberKnownClients(int _numberKnownClients) { numberKnownClients = _numberKnownClients; };
  void setClientName(int clientNumber, const char *name) {
    if (clientNumber < MAX_NUMBER_CLIENTS_DISPLAYED) {
      strncpy(clientName[clientNumber], name, MAX_CLIENT_NAME_LEN);
    }
  };
  void addLogMessage(const char *message);
  void showLog(void);
private:
  ScreenMode currentScreen;
  char clockName[MAX_CLOCK_NAME_LEN];
  char clockSpeed[MAX_CLOCK_SPEED_LEN];
  //int msPerModelSecond; // milliseconds each model second takes. A 500 means the clock runs twice as fast as real time.
  int clockHour;
  int clockMinute;
  char clockWeekday[MAX_CLOCK_WEEKDAY_LEN];
  bool clockHalted;
  int numberKnownClients;
  char clientName[MAX_NUMBER_CLIENTS_DISPLAYED][MAX_CLIENT_NAME_LEN];
  char logLine[MAX_NUMBER_LOG_LINES][MAX_LOG_MESSAGE_LEN];
  int numberLogLines;
  void setSmallTextSize(void);
  int getTextHeight(void);
  int getTextCharsPerLine(void);
  void setNormalTextSize(void);
  void setLargeTextSize(void);
};
#endif
