#ifndef fastclock_h_included
#define fastclock_h_included

#include "config.h"
#include "display.h"
#include "radio.h"

#define MIN_TIME_ms_BETWEEN_CLOCK_UPDATES 200
#define DEFAULT_ms_PER_MODEL_SECOND 250

class Fastclock {
public:
  Fastclock(Display *d, Radio *r): display(d), radio(r) { weekday=0; hour=0; minute=0; second=0; millisecond=0; msPerModelSecond=DEFAULT_ms_PER_MODEL_SECOND; halted = true; };
  void begin(void);
  void loop(void);
  void incrementClockByMilliseconds(int amount);
  void setClockSpeed(unsigned int msPerModelSecond);
  void setTime(uint8_t hours, uint8_t minutes) { hour = hours; minute = minutes; second = 0; millisecond = 0; };
  void setTime(uint8_t hours, uint8_t minutes, uint8_t seconds) { hour = hours; minute = minutes; second = seconds; millisecond = 0; };
  void setTime(uint8_t hours, uint8_t minutes, uint8_t seconds, uint16_t milliseconds) { hour = hours; minute = minutes; second = seconds; millisecond = milliseconds; };
  void setWeekday(uint8_t _weekday) { weekday = _weekday; };
  void setClockHalted(bool setToHalt) { halted = setToHalt; display->setClockHalted(halted); };
private:
  Display *display;
  Radio *radio;
  unsigned long lastTimeTickUsed; // used to calculate model time
  unsigned long msPerModelSecond; // 500 = twice as fast as real time
  uint8_t weekday;
  uint8_t hour;
  uint8_t minute;
  uint8_t second;
  uint16_t millisecond;
  bool halted;

};

#endif
